MODULE trdmld_rst
   !!=================================================================================
   !!                       ***  MODULE  trdmld_rst  ***
   !! Ocean dynamic :  Input/Output files for restart on mixed-layer diagnostics
   !!=================================================================================
   !! History :  9.0  ! 05-05 (C. Deltel) Original code
   !!---------------------------------------------------------------------------------
#if defined key_trdmld
   !!---------------------------------------------------------------------------------
   USE dom_oce         ! ocean space and time domain
   USE trdmod_oce      ! ocean variables for trend diagnostics (i.e. icp/mixed-layer/vorticity)
   USE in_out_manager  ! I/O manager
   USE daymod          ! calendar
   USE ioipsl          !

   IMPLICIT NONE
   PRIVATE
  
   PUBLIC   trd_mld_rst_read    ! routine called by trd_mld_init
   PUBLIC   trd_mld_rst_write   ! routine called by step.F90
  
   CHARACTER (len=48) ::   crestart = 'initial.nc'   ! restart file name
   !!---------------------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2005) 
   !!---------------------------------------------------------------------------------
  
CONTAINS
  
   SUBROUTINE trd_mld_rst_write( kt )
      !!--------------------------------------------------------------------------------
      !!                  ***  SUBROUTINE trd_mld_rst_wri  ***
      !!                
      !! ** Purpose :   Write mixed-layer diagnostics restart fields.
      !!-----------------------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index
      !!    
      LOGICAL ::   llbon
      CHARACTER (len=50) ::   clname, cln
      REAL(wp) ::   zdate0
      REAL(wp), DIMENSION(1) :: zdept
      INTEGER ::   ic, jc, itime, inumwrs_mld
      !!-----------------------------------------------------------------------------
    
      IF( ( mod( kt, nstock ) == 0 ) .OR. ( kt == nitend ) ) THEN
             
         inumwrs_mld = 47
         !-- Delete the restart file if it exists 
         INQUIRE( FILE=crestart, EXIST=llbon )
         IF(llbon) THEN
            OPEN( UNIT=inumwrs_mld, FILE=crestart, STATUS='old' )
            CLOSE( inumwrs_mld, STATUS='delete' )
         ENDIF

         !-- Name of the new restart file
         ic = 1
         DO jc = 1, 16
            IF( cexper(jc:jc) /= ' ' )   ic = jc
         END DO
         WRITE(cln,'("_",i4.4,i2.2,i2.2,"_restart_mld")') nyear, nmonth, nday
         clname = cexper(1:ic)//cln
         ic = 1
         DO jc = 1, 48
            IF( clname(jc:jc) /= ' ' ) ic = jc
         END DO
         crestart = clname(1:ic)//".nc"
         itime = 0
         CALL ymds2ju( nyear, nmonth, nday, 0.e0, zdate0 )

         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'trdmld_rst: output for ML diags. restart, with trd_mld_rst_write routine'
            WRITE(numout,*) '~~~~~~~~~~'
            WRITE(numout,*) '            in file : ', TRIM(crestart), ' at it= ', kt, ' date= ', ndastp
            WRITE(numout,*)
         ENDIF

         !-- Create the NetCDF restart file and write tje appropriate fields
         !   N.B. In this section, 3rd dimension in arrays is NOT depth 
         zdept(1) = 1.
         CALL restini( 'NONE', jpi, jpj, glamt, gphit, 1, zdept, clname,   &
            &          itime, zdate0, rdt*nstock ,inumwrs_mld, domain_id=nidom )
       
         IF( ln_trdmld_instant ) THEN 
            CALL restput( inumwrs_mld, 'tmlbb'           , jpi, jpj,      1, 0, tmlbb           )
            CALL restput( inumwrs_mld, 'tmlbn'           , jpi, jpj,      1, 0, tmlbn           )
            CALL restput( inumwrs_mld, 'tmlatfb'         , jpi, jpj,      1, 0, tmlatfb         )

            CALL restput( inumwrs_mld, 'smlbb'           , jpi, jpj,      1, 0, smlbb           )
            CALL restput( inumwrs_mld, 'smlbn'           , jpi, jpj,      1, 0, smlbn           )
            CALL restput( inumwrs_mld, 'smlatfb'         , jpi, jpj,      1, 0, smlatfb         )
         ELSE
            CALL restput( inumwrs_mld, 'rmldbn'          , jpi, jpj,      1, 0, rmldbn          )

            !-- Temperature
            CALL restput( inumwrs_mld, 'tmlbn'           , jpi, jpj,      1, 0, tmlbn           )
            CALL restput( inumwrs_mld, 'tml_sumb'        , jpi, jpj,      1, 0, tml_sumb        )
            CALL restput( inumwrs_mld, 'tmltrd_csum_ub'  , jpi, jpj, jpltrd, 0, tmltrd_csum_ub  )
            CALL restput( inumwrs_mld, 'tmltrd_atf_sumb' , jpi, jpj,      1, 0, tmltrd_atf_sumb )

            !-- Salinity
            CALL restput( inumwrs_mld, 'smlbn'           , jpi, jpj,      1, 0, smlbn           )
            CALL restput( inumwrs_mld, 'sml_sumb'        , jpi, jpj,      1, 0, sml_sumb        )
            CALL restput( inumwrs_mld, 'smltrd_csum_ub'  , jpi, jpj, jpltrd, 0, smltrd_csum_ub  )
            CALL restput( inumwrs_mld, 'smltrd_atf_sumb' , jpi, jpj,      1, 0, smltrd_atf_sumb )
         ENDIF
         !
         CALL restclo( inumwrs_mld )
         ! 
      ENDIF
      !    
   END SUBROUTINE trd_mld_rst_write


   SUBROUTINE trd_mld_rst_read
    !!----------------------------------------------------------------------------
    !!                   ***  SUBROUTINE trd_mld_rst_lec  ***
    !!                   
    !! ** Purpose :   Read file for mixed-layer diagnostics restart.
    !!----------------------------------------------------------------------------
    LOGICAL ::   llog
    REAL(wp) ::  zlamt(jpi,jpj), zphit(jpi,jpj)
    CHARACTER (len=8 ) ::   clvnames(30)
    CHARACTER (len=32) ::   clname = 'restart_mld'
    INTEGER  ::  itime, ibvar, &  
                 inum  ! temporary logical unit
    REAL(wp) ::   zdate0, zdt
    REAL(wp), DIMENSION(1) :: zdept
    !!-----------------------------------------------------------------------------

    IF(lwp)  THEN
       WRITE(numout,*)
       WRITE(numout,*) ' trd_mld_rst_read : read the NetCDF MLD restart file'
       WRITE(numout,*) ' ~~~~~~~~~~~~~~~~'
    ENDIF

    itime = 0
    llog  = .FALSE.
    zlamt(:,:) = 0.e0   ;   zphit(:,:) = 0.e0   ;   zdept(1)   = 0.e0
    CALL restini( clname, jpi, jpj, zlamt, zphit, 1, zdept, 'NONE',   &
       &          itime, zdate0, zdt, inum, domain_id=nidom )
  
    CALL ioget_vname( inum, ibvar, clvnames)

    IF(lwp) THEN
       WRITE(numout,*) 
       WRITE(numout,*) '                    Info on the MLD restart file read : '
       WRITE(numout,*) '                      File name           : ', clname
       WRITE(numout,*) '                      number of variables : ', ibvar
       WRITE(numout,*) '                      NetCDF variables    : ', clvnames
       WRITE(numout,*)
    ENDIF

    IF( ln_trdmld_instant ) THEN 
       CALL restget( inum, 'tmlbb'           , jpi, jpj,      1, 0, llog, tmlbb          )
       CALL restget( inum, 'tmlbn'           , jpi, jpj,      1, 0, llog, tmlbn          )
       CALL restget( inum, 'tmlatfb'         , jpi, jpj,      1, 0, llog, tmlatfb        )

       CALL restget( inum, 'smlbb'           , jpi, jpj,      1, 0, llog, smlbb          )
       CALL restget( inum, 'smlbn'           , jpi, jpj,      1, 0, llog, smlbn          )
       CALL restget( inum, 'smlatfb'         , jpi, jpj,      1, 0, llog, smlatfb        )
    ELSE
       CALL restget( inum, 'rmldbn'          , jpi, jpj,      1, 0, llog, rmldbn         ) ! needed for rmld_sum

       !-- Temperature
       CALL restget( inum, 'tmlbn'           , jpi, jpj,      1, 0, llog, tmlbn          ) ! needed for tml_sum
       CALL restget( inum, 'tml_sumb'        , jpi, jpj,      1, 0, llog, tml_sumb       )
       CALL restget( inum, 'tmltrd_csum_ub'  , jpi, jpj, jpltrd, 0, llog, tmltrd_csum_ub )
       CALL restget( inum, 'tmltrd_atf_sumb' , jpi, jpj,      1, 0, llog, tmltrd_atf_sumb)

       !-- Salinity
       CALL restget( inum, 'smlbn'           , jpi, jpj,      1, 0, llog, smlbn          ) ! needed for sml_sum
       CALL restget( inum, 'sml_sumb'        , jpi, jpj,      1, 0, llog, sml_sumb       )
       CALL restget( inum, 'smltrd_csum_ub'  , jpi, jpj, jpltrd, 0, llog, smltrd_csum_ub )
       CALL restget( inum, 'smltrd_atf_sumb' , jpi, jpj,      1, 0, llog, smltrd_atf_sumb)

       CALL restclo( inum )
    ENDIF

  END SUBROUTINE trd_mld_rst_read
  
#else
  !!=================================================================================
  !!                       ***  MODULE  trdmld_rst  ***
  !! Ocean dynamic :  Input/Output files for restart on mixed-layer diagnostics
  !!=================================================================================
CONTAINS
  SUBROUTINE trd_mld_rst_write( kt )           !  No ML diags ==> empty routine
    WRITE(*,*) 'trd_mld_rst_wri: You should not have seen this print! error?', kt
  END SUBROUTINE trd_mld_rst_write
  SUBROUTINE trd_mld_rst_read                  !  No ML Diags ==> empty routine
  END SUBROUTINE trd_mld_rst_read
#endif

  !!=================================================================================
END MODULE trdmld_rst
